@@ -4,5 +4,4 @@
 (**************************************************************)
 open Trans
 
-val oldf : Mbuf.t -> ('a,'b,'c,'d) Appl_intf.Old.power -> Appl_intf.Old.t
-val newf : Mbuf.t -> 'a Appl_intf.New.power -> Appl_intf.New.t
+val newf : 'a Appl_intf.New.power -> Appl_intf.New.t
