@@ -5,7 +5,7 @@ echo "Creating:"
 #
 # Don't do this for config.mk and clean_all.mk, they contain NT specific options
 #
-for i in mk/files mk/ocaml mk/ocamlopt mk/preamble mk/sub; do
+for i in mk/files mk/ocaml mk/ocamlopt mk/preamble mk/rules mk/sub; do
       echo "  " $i
       sed -f tools/ntify.sed $i.mk > $i.nmk
 done
@@ -20,7 +20,8 @@ done
 # Don't do this for maestro and ejava: they have their own build process.
 #
 for i in appl mm crypto crypto/isaac crypto/OpenSSL doc doc/layers groupd \
-    infr layers mk route socket socket/s socket/u trans tools type util cejava; do
+    infr layers mk route socket socket/u socket/s socket/s/unix socket/s/nt \
+    tar trans tools type util cejava; do
       echo "  " $i/Makefile
       sed -f tools/ntify.sed $i/Makefile > $i/Makefile.nt
 done
@@ -31,7 +32,3 @@ for i in bypass gossip other security transis debug total vsync \
       sed -f tools/ntify.sed layers/$i/Makefile > layers/$i/Makefile.nt
 done
 
-#
-# take care of the dependencies
-echo "   .depend" 
-sed -f tools/ntify.sed .depend > .depend.nt
